/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2005-2008. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("net.jazz.ajax.ui.PageContainer");

dojo.require("dijit._Widget");

dojo.require("net.jazz.ajax.core.internal.util");
dojo.require("net.jazz.ajax.ui.ErrorPage");

dojo.declare("net.jazz.ajax.ui.PageContainer", dijit._Widget, {
	
	// summary:
	//		a widget for displaying pages
	// usage notes:
	//		this widget contains no public methods; simply instantiate it and
	//		place it within a Jazz Framework-based application
	
	//
	// Internal and lifecycle methods; do not call directly
	//
	
	create: function(args, srcNodeRef) {
		if(!srcNodeRef) {
			srcNodeRef = document.createElement("div");
		}
		// subscribe to events
		
		net.jazz.ajax.ui.PageContainer.superclass.create.call(this, args, srcNodeRef);
	},
	
	postCreate: function() {
		this.domNode.id = "net-jazz-ajax-PageContainer";

		var outerWrapper = document.createElement("div");
		outerWrapper.className = "net-jazz-ajax-PageContainer-RightBorder";
  
		var innerWrapper = document.createElement("div");
		innerWrapper.className = "net-jazz-ajax-PageContainer-LeftBorder";
		
		var contentArea = document.createElement("div");
		contentArea.className = "net-jazz-ajax-PageContainer-ContentArea";
		
		innerWrapper.appendChild(contentArea)
		outerWrapper.appendChild(innerWrapper);
	
		this.domNode.appendChild(outerWrapper);
		
		this.contentArea = contentArea;
		
		// subscribe to relevant events
		dojo.subscribe(net.jazz.ajax.internal.events.viewPage, this, "_viewPage");
		dojo.subscribe(net.jazz.ajax.internal.events.viewErrorPage, this, "_viewErrorPage");
		
		// preload important net.jazz.ajax images
		setTimeout(dojo.hitch(this,this._preloadImages),0);
	},
	
	_viewPage: function(page) {
		// if we're already viewing the incoming page, bail out
		if(this._currentPage && this._currentPage === page.domNode) {
			return;
		}
		
		// otherwise display the page
		if(this._currentPage) {
			this.contentArea.replaceChild(page.domNode, this._currentPage);
		} else {
			this.contentArea.appendChild(page.domNode);
		}
		
		this._currentPage = page.domNode;
	},
	
	_viewErrorPage: function(message, finePrint) {
		if (!this._errorPage) 
			this._errorPage = new net.jazz.ajax.ui.ErrorPage();
		
		this._errorPage.setMessage(message, finePrint);
		
		if (this._currentPage == this._errorPage.domNode) return;
		
		if(this._currentPage)
			this.contentArea.replaceChild(this._errorPage.domNode, this._currentPage);
		else
			this.contentArea.appendChild(this._errorPage.domNode);
		
		this._currentPage = this._errorPage.domNode;
	},
	
	_preloadImages: function() {
		if (this.pictures) return;
		this.pictures = new Array();
		this.pictures.push((new Image()).src = dojo.moduleUrl("net.jazz.ajax", "ui/internal/images/button/sm-button-left.gif"));
		this.pictures.push((new Image()).src = dojo.moduleUrl("net.jazz.ajax", "ui/internal/images/button/sm-button-right.gif"));
		this.pictures.push((new Image()).src = dojo.moduleUrl("net.jazz.ajax", "ui/internal/images/button/sm-button-mid.gif"));
		this.pictures.push((new Image()).src = dojo.moduleUrl("net.jazz.ajax", "ui/internal/images/button/lg-button-left.gif"));
		this.pictures.push((new Image()).src = dojo.moduleUrl("net.jazz.ajax", "ui/internal/images/button/lg-button-right.gif"));
		this.pictures.push((new Image()).src = dojo.moduleUrl("net.jazz.ajax", "ui/internal/images/button/lg-button-mid.gif"));
		this.pictures.push((new Image()).src = dojo.moduleUrl("net.jazz.ajax", "ui/internal/images/button/menu-arrow.gif"));
		this.pictures.push((new Image()).src = dojo.moduleUrl("net.jazz.ajax", "ui/internal/images/lp-sctn-hdr-left.gif"));
		this.pictures.push((new Image()).src = dojo.moduleUrl("net.jazz.ajax", "ui/internal/images/lp-sctn-hdr-mid.gif"));
		this.pictures.push((new Image()).src = dojo.moduleUrl("net.jazz.ajax", "ui/internal/images/lp-sctn-hdr-right.gif"));
		this.pictures.push((new Image()).src = dojo.moduleUrl("net.jazz.ajax", "ui/internal/images/chev-dwn.gif"));
		this.pictures.push((new Image()).src = dojo.moduleUrl("net.jazz.ajax", "ui/internal/images/sash-arrow-left-close.gif"));
		this.pictures.push((new Image()).src = dojo.moduleUrl("net.jazz.ajax", "ui/internal/images/twistie-open.gif"));
		this.pictures.push((new Image()).src = dojo.moduleUrl("net.jazz.ajax", "ui/internal/images/twistie-3quarter.gif"));
		
		//defer loading of hover state images for 4 seconds 
		setTimeout(dojo.hitch(this,this._preloadSecondaryImages),4000);
	},
	
	_preloadSecondaryImages: function() {
		this.pictures.push((new Image()).src = dojo.moduleUrl("net.jazz.ajax", "ui/internal/images/button/sm-button-left-hov.gif"));
		this.pictures.push((new Image()).src = dojo.moduleUrl("net.jazz.ajax", "ui/internal/images/button/sm-button-right-hov.gif"));
		this.pictures.push((new Image()).src = dojo.moduleUrl("net.jazz.ajax", "ui/internal/images/button/sm-button-mid-hov.gif"));
		this.pictures.push((new Image()).src = dojo.moduleUrl("net.jazz.ajax", "ui/internal/images/button/lg-button-left-hov.gif"));
		this.pictures.push((new Image()).src = dojo.moduleUrl("net.jazz.ajax", "ui/internal/images/button/lg-button-right-hov.gif"));
		this.pictures.push((new Image()).src = dojo.moduleUrl("net.jazz.ajax", "ui/internal/images/button/lg-button-mid-hov.gif"));		
		this.pictures.push((new Image()).src = dojo.moduleUrl("net.jazz.ajax", "ui/internal/images/button/menu-arrow-hov.gif"));
		this.pictures.push((new Image()).src = dojo.moduleUrl("net.jazz.ajax", "ui/internal/images/chev-dwn-hov.gif"));
		this.pictures.push((new Image()).src = dojo.moduleUrl("net.jazz.ajax", "ui/internal/images/twistie-closed.gif"));
		this.pictures.push((new Image()).src = dojo.moduleUrl("net.jazz.ajax", "ui/internal/images/lp-sctn-hdr-left-hov.gif"));
		this.pictures.push((new Image()).src = dojo.moduleUrl("net.jazz.ajax", "ui/internal/images/lp-sctn-hdr-mid-hov.gif"));
		this.pictures.push((new Image()).src = dojo.moduleUrl("net.jazz.ajax", "ui/internal/images/lp-sctn-hdr-right-hov.gif"));
	}
});