/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2005-2008. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
(function() {
dojo.provide("com.ibm.team.repository.web.ui.internal.admin.server.ServerPage");

dojo.require("dijit._Widget");
dojo.require("dojox.data.dom");

dojo.require("net.jazz.ajax.ui.PlatformUI");
dojo.require("net.jazz.ajax.ui.SideNavigationLayout");

dojo.require("com.ibm.team.repository.web.ui.internal.admin.server.ActiveServicesView");
dojo.require("com.ibm.team.repository.web.ui.internal.admin.server.ComponentsView");
dojo.require("com.ibm.team.repository.web.ui.internal.admin.server.ConfigureAdvanced");
dojo.require("com.ibm.team.repository.web.ui.internal.admin.server.ConfigureDatabaseConnection");
dojo.require("com.ibm.team.repository.web.ui.internal.admin.server.ConfigureFeedSettings");
dojo.require("com.ibm.team.repository.web.ui.internal.admin.server.ConfigureMailSettings");
dojo.require("com.ibm.team.repository.web.ui.internal.admin.server.LicenseEditor");
dojo.require("com.ibm.team.repository.web.ui.internal.admin.server.ReloadLogSettings");
dojo.require("com.ibm.team.repository.web.ui.internal.admin.server.ServerStatus");
dojo.require("com.ibm.team.repository.web.ui.internal.admin.server.Statistics");
dojo.require("com.ibm.team.repository.web.ui.internal.admin.server.ViewCounters");
dojo.require("com.ibm.team.repository.web.ui.internal.admin.server.ProvisionStatus");
dojo.require("com.ibm.team.repository.web.ui.internal.admin.server.ServerReset");

dojo.declare("com.ibm.team.repository.web.ui.internal.admin.server.ServerPage", dijit._Widget, {
	
	//
	// Methods intended to be called by child pages
	//
	
	reloadCurrent: function() {
		this._previousActionEntry = this._currentActionEntry || null;
		this._showCurrentWidget(/*isReload*/true);
	},	
	
	//
	// Dojo lifecycle and internal methods
	//
	
	postCreate: function() {
		// call superclass's postCreate method
		this.inherited(arguments);
		
		// initialize widget cache
		this._widgetCache = {};
		
		// set the root style for the Server page and all of its sub-pages
		this.domNode.className = "com-ibm-team-repository-admin";
		
		// instantiate the side-nav layout UI and append it to the page
		this._appView = new net.jazz.ajax.ui.SideNavigationLayout();
		this.domNode.appendChild(this._appView.domNode);
		
		// register actions and create side-nav hyperlinks
		ActionRegistry = getWorkbench().getActionRegistry();
		ActionRegistry.registerAction(PREFIX + "_defaultAction", this, "_defaultAction");
		
		var self = this;
		this.allLinks = [];
		dojo.forEach(ACTION_SETS, function(actionSet) {
			var navSection = document.createElement("div");
			self.connect(navSection, "onclick", self._onNavSectionClick);
			navSection.className = "com-ibm-team-repository-admin-navSection";
			if(actionSet.label !== "Internal Tools" || location.search.indexOf("internal") !== -1) {
				self._appView.addNavigationSection(navSection, actionSet.label, true);
			}
			dojo.forEach(actionSet.actions, function(action){
				var id = PREFIX + action.shortId;
				ACTION_MAP[action.shortId] = action;
				var link = getNavigationLink(id, action.label);
				link.__jazzActionId = action.shortId;
				navSection.appendChild(link);
				var span = document.createElement("span");
				span.__jazzReload = true;
				link.appendChild(span);
				self.allLinks.push(link);
				ActionRegistry.registerAction(id, self, "_handleAction");
			});
		});
	},
	
	_defaultAction: function() {
		// by default, show the first section from the left-nav
		this._handleAction({action: PREFIX + ACTION_SETS[0].actions[0].shortId});
	},
	
	_handleAction: function(args) {
		if (!isAdmin()) {
			this._showUnauthorized();
			return;
		}

		if(!args.action) {
			console.debug("ServerPage.js#_handleAction: missing required param 'args.action'");
			return;
		}
		
		// cache the previous entry to see if the widget should be destroyed or
		// not
		this._previousActionEntry = this._currentActionEntry || null;
		
		// parse out the short id from the fully-qualified id
		var shortId = args.action.substring(args.action.lastIndexOf(".") + 1);
		this._currentActionEntry = ACTION_MAP[shortId];
		this._currentArgs = args;
		this._currentArgs.parent = this;
		
		// set the title for the action
		getWorkbench().setTitle(this._currentActionEntry.label);
		
		// highlight the selected link and unhighlight the others
		dojo.forEach(this.allLinks, function(link){
			link.className = link.__jazzActionId === shortId ? "selected" : "";
		});
		
		// call the actual method implementation
		this._showCurrentWidget();
	},
	
	_onNavSectionClick: function(e) {
		if(e.target.__jazzReload) {
			dojo.stopEvent(e);
			this.reloadCurrent();
		}
	},
	
	_showCurrentWidget: function(isReload) {
		// save a reference to the prior content widget
		this._previousContentWidget = this._currentContentWidget || null;
		
		if(this._currentActionEntry.cached) {
			this._showCachedWidget(isReload);
		} else {
			this._showUncachedWidget();
		}
		
		// help the garbage collector do its job
		if(this._previousContentWidget && !this._previousActionEntry.cached) {
			this._previousContentWidget.destroyRecursive();
		}
	},
	
	_showCachedWidget: function(isReload) {
		var className = this._currentActionEntry.widget;
		if(this._widgetCache[className]) {
			// get widget from the cache if it exists
			this._currentContentWidget = this._widgetCache[className];
		} else {
			// otherwise instantiate it and add it to the cache
			var constructor = dojo.getObject(SERVER_PACKAGE + className);
			this._currentContentWidget = new constructor(this._currentArgs);
			dojo.addClass(this._currentContentWidget.domNode, "com-ibm-team-repository-admin-contentArea");
			this._widgetCache[className] = this._currentContentWidget;
		}
		if(isReload) {
			this._currentContentWidget.reload(this._currentArgs);
		} else {
			this._currentContentWidget.show(this._currentArgs);
			this._appView.setMainContent(this._currentContentWidget.domNode);
		}
	},
	
	_showUncachedWidget: function() {
		var constructor = dojo.getObject(SERVER_PACKAGE + this._currentActionEntry.widget);
		this._currentContentWidget = new constructor(this._currentArgs);
		dojo.addClass(this._currentContentWidget.domNode, "com-ibm-team-repository-admin-contentArea");
		this._appView.setMainContent(this._currentContentWidget.domNode);
	},
	
	_showUnauthorized: function(){
		var div = document.createElement("div");
		div.className = "com-ibm-team-repository-admin-noAuth";
		div.innerHTML = "You are not authorized to access the Jazz Team Server Admin UI";
		dojox.data.dom.removeChildren(this.domNode);
		this.domNode.appendChild(div);
	}
});

//
// aliases, constants, and module-private variables
//

var NoAuth         = com.ibm.team.repository.web.ui.internal.admin.NoAuth;
var PlatformUI     = net.jazz.ajax.ui.PlatformUI;
var ViewCounters   = com.ibm.team.repository.web.ui.internal.admin.server.ViewCounters;

var ActionRegistry = null;
var PREFIX         = "com.ibm.team.repository.admin.";
var SERVER_PACKAGE = "com.ibm.team.repository.web.ui.internal.admin.server.";
var ACTION_MAP     = {};

//
// The following defines the left-nav links and their bindings to actions and
// widgets.
//

var ACTION_SETS = [
	{ label: "Status", actions: [
		{label: "Status Summary", shortId: "serverStatus", widget: "ServerStatus"},
		{label: "Statistics", shortId: "statistics", widget: "Statistics"},
		{label: "Component Status", shortId: "componentStatus", widget: "ComponentsView"},
		{label: "Active Services", shortId: "activeServices", widget: "ActiveServicesView"}
	]},
	{ label: "Configuration", actions: [
		{label: "E-mail Settings", shortId: "configureMailSettings", widget: "ConfigureMailSettings"},
		{label: "Database Connection", shortId: "configureDatabaseConnection", widget: "ConfigureDatabaseConnection"},
		{label: "Feed Settings", shortId: "configureFeedSettings", widget: "ConfigureFeedSettings"},
		{label: "License Key Management", shortId: "manageLicenses", widget: "LicenseEditor", cached: true},
		{label: "Advanced Properties", shortId: "configureAdvanced", widget: "ConfigureAdvanced"}
	]},
	{ label: "Internal Tools", actions: [
		{label: "Counters", shortId: "viewCounters", widget: "ViewCounters", cached: true},
		{label: "Reload Log Settings", shortId: "reloadLoggingSettings", widget: "ReloadLogSettings"},
		{label: "Provision Status", shortId: "provisionStatus", widget: "ProvisionStatus"},
		{label: "Server Reset", shortId: "serverReset", widget: "ServerReset"}
	]}
];

function getNavigationLink(action, title) {
	var a = document.createElement("a");
	a.href = "#action=" + action;
	a.appendChild(document.createTextNode(title));
	
	return a;
}

function getWorkbench() {
	return net.jazz.ajax.ui.PlatformUI.getWorkbench();
}

function isAdmin() {
	return com.ibm.team.repository.web.noAdminAuth ? false : true;
}

})();